#!/usr/bin/env python
# -*- coding: utf-8 -*-

# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
# ------------
#| Change Log |
# ------------
# Rel 0.1: Initial relase http://gimpchat.com/viewtopic.php?f=9&t=11524
# Rel 0.2: change procedure Gmic to Gmic-QT
# Rel 0.3: fixing some bugs
# Rel 0.4: separation of Gmic filters, add frame.
# Rel 0.5: change name in menu (old name: File/Create/Logos/G'mic Logo..), add option: use for Logo or Image, use color or patterns, Random, save parameters as layer.
# Rel 0.6: added tabs, updated routines for Gmic-QT +3.2.6, added new GMIC filters (now 80 command), rename layer to descriptive

############################################################################################################################################################
########################################################## START gimpfu3 by J.F.Garcia [arakne] ############################################################
############################################################################################################################################################
#   Gimp-Python - allows the writing of GIMP plug-ins in Python.
#   Copyright (C) 1997  James Henstridge <james@daa.com.au>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import time

import string as _string
import math
import gimp
import gimpcolor
import re
from gimpenums import *
pdb = gimp.pdb

import gettext
t = gettext.translation("gimp20-python", gimp.locale_directory, fallback=True)
_ = t.ugettext

class error(RuntimeError): pass
class CancelError(RuntimeError): pass

PF_INT8        = PDB_INT8
PF_INT16       = PDB_INT16
PF_INT32       = PDB_INT32
PF_INT         = PF_INT32
PF_FLOAT       = PDB_FLOAT
PF_STRING      = PDB_STRING
PF_VALUE       = PF_STRING
PF_COLOR       = PDB_COLOR
PF_COLOUR      = PF_COLOR
PF_ITEM        = PDB_ITEM
PF_DISPLAY     = PDB_DISPLAY
PF_IMAGE       = PDB_IMAGE
PF_LAYER       = PDB_LAYER
PF_CHANNEL     = PDB_CHANNEL
PF_DRAWABLE    = PDB_DRAWABLE
PF_VECTORS     = PDB_VECTORS

PF_TOGGLE      = 1000
PF_BOOL        = PF_TOGGLE
PF_SLIDER      = 1001
PF_SPINNER     = 1002
PF_ADJUSTMENT  = PF_SPINNER

PF_FONT        = 1003
PF_FILE        = 1004
PF_BRUSH       = 1005
PF_PATTERN     = 1006
PF_GRADIENT    = 1007
PF_RADIO       = 1008
PF_TEXT        = 1009
PF_PALETTE     = 1010
PF_FILENAME    = 1011
PF_DIRNAME     = 1012
PF_OPTION      = 1013
PF_TAB         = 1014
PF_DRAW        = 1015
PF_TITLE       = 1016

_type_mapping = {
    PF_INT8        : PDB_INT8,
    PF_INT16       : PDB_INT16,
    PF_INT32       : PDB_INT32,
    PF_FLOAT       : PDB_FLOAT,
    PF_STRING      : PDB_STRING,
    PF_COLOR       : PDB_COLOR,
    PF_ITEM        : PDB_ITEM,
    PF_DISPLAY     : PDB_DISPLAY,
    PF_IMAGE       : PDB_IMAGE,
    PF_LAYER       : PDB_LAYER,
    PF_CHANNEL     : PDB_CHANNEL,
    PF_DRAWABLE    : PDB_DRAWABLE,
    PF_VECTORS     : PDB_VECTORS,

    PF_TOGGLE      : PDB_INT32,
    PF_SLIDER      : PDB_FLOAT,
    PF_SPINNER     : PDB_INT32,

    PF_FONT        : PDB_STRING,
    PF_FILE        : PDB_STRING,
    PF_BRUSH       : PDB_STRING,
    PF_PATTERN     : PDB_STRING,
    PF_GRADIENT    : PDB_STRING,
    PF_RADIO       : PDB_STRING,
    PF_TEXT        : PDB_STRING,
    PF_PALETTE     : PDB_STRING,
    PF_FILENAME    : PDB_STRING,
    PF_DIRNAME     : PDB_STRING,
    PF_OPTION      : PDB_INT32,
    PF_TAB         : PDB_STRING,
    PF_DRAW        : PDB_INT32,
    PF_TITLE       : PDB_STRING,
}

_obj_mapping = {
    PF_INT8        : int,
    PF_INT16       : int,
    PF_INT32       : int,
    PF_FLOAT       : float,
    PF_STRING      : str,
    PF_COLOR       : gimpcolor.RGB,
    PF_ITEM        : int,
    PF_DISPLAY     : gimp.Display,
    PF_IMAGE       : gimp.Image,
    PF_LAYER       : gimp.Layer,
    PF_CHANNEL     : gimp.Channel,
    PF_DRAWABLE    : gimp.Drawable,
    PF_VECTORS     : gimp.Vectors,

    PF_TOGGLE      : bool,
    PF_SLIDER      : float,
    PF_SPINNER     : int,

    PF_FONT        : str,
    PF_FILE        : str,
    PF_BRUSH       : str,
    PF_PATTERN     : str,
    PF_GRADIENT    : str,
    PF_RADIO       : str,
    PF_TEXT        : str,
    PF_PALETTE     : str,
    PF_FILENAME    : str,
    PF_DIRNAME     : str,
    PF_OPTION      : int,
    PF_TAB         : str,
    PF_DRAW        : int,
    PF_TITLE       : str,
}

_registered_plugins_ = {}

def register(proc_name, blurb, help, author, copyright, date, label, imagetypes, params, results, function, menu=None, domain=None, on_query=None, on_run=None):
    """This is called to register a new plug-in."""

    # First perform some sanity checks on the data
    def letterCheck(str):
        allowed = _string.letters + _string.digits + "_" + "-"
        for ch in str:
            if not ch in allowed:
		return 0
        else:
            return 1

    if not letterCheck(proc_name):
        raise error, "procedure name contains illegal characters"

    for ent in params:
        if len(ent) < 4:
            raise error, ("parameter definition must contain at least 4 "
                          "elements (%s given: %s)" % (len(ent), ent))

        if type(ent[0]) != int:
            raise error, "parameter types must be integers"

        if not letterCheck(ent[1]):
            raise error, "parameter name contains illegal characters"

    for ent in results:
        if len(ent) < 3:
            raise error, ("result definition must contain at least 3 elements (%s given: %s)" % (len(ent), ent))

        if type(ent[0]) != type(42):
            raise error, "result types must be integers"

        if not letterCheck(ent[1]):
            raise error, "result name contains illegal characters"

    plugin_type = PLUGIN

    if (not proc_name.startswith("python-") and
        not proc_name.startswith("python_") and
        not proc_name.startswith("extension-") and
        not proc_name.startswith("extension_") and
        not proc_name.startswith("plug-in-") and
        not proc_name.startswith("plug_in_") and
        not proc_name.startswith("file-") and
        not proc_name.startswith("file_")):
           proc_name = "python-fu-" + proc_name

    # if menu is not given, derive it from label
    need_compat_params = False
    if menu is None and label:
        fields = label.split("/")
        if fields:
            label = fields.pop()
            menu = "/".join(fields)
            need_compat_params = True

            import warnings
            message = ("%s: passing the full menu path for the menu label is "
                       "deprecated, use the 'menu' parameter instead" % (proc_name))
            warnings.warn(message, DeprecationWarning, 3)

        if need_compat_params and plugin_type == PLUGIN:
            file_params = [(PDB_STRING, "filename", "The name of the file", ""),
                           (PDB_STRING, "raw-filename", "The name of the file", "")]

            if menu is None:
                pass
            elif menu.startswith("<Load>"):
                params[0:0] = file_params
            elif menu.startswith("<Image>") or menu.startswith("<Save>"):
                params.insert(0, (PDB_IMAGE, "image", "Input image", None))
                params.insert(1, (PDB_DRAWABLE, "drawable", "Input drawable", None))
                if menu.startswith("<Save>"):
                    params[2:2] = file_params

    _registered_plugins_[proc_name] = (blurb, help, author, copyright, date, label, imagetypes, plugin_type, params, results, function, menu, domain, on_query, on_run)

def _query():
    for plugin in _registered_plugins_.keys():
        (blurb, help, author, copyright, date, label, imagetypes, plugin_type, params, results, function, menu, domain, on_query, on_run) = _registered_plugins_[plugin]

        def make_params(params):
            return [(_type_mapping[x[0]], x[1], _string.replace(x[2], "_", "")) for x in params]

        params = make_params(params)
        # add the run mode argument ...
        params.insert(0, (PDB_INT32, "run-mode", "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }"))

        results = make_params(results)

        if domain:
            try:
                (domain, locale_dir) = domain
                gimp.domain_register(domain, locale_dir)
            except ValueError:
                gimp.domain_register(domain)

        gimp.install_procedure(plugin, blurb, help, author, copyright, date, label, imagetypes, plugin_type, params, results)
        if menu:
            gimp.menu_register(plugin, menu)
        if on_query:
            on_query()

def getXtra(xtra, valName, defValue):
    p = defValue
    if xtra.get(valName):
        p = xtra.get(valName)
    return p

def _get_defaults(proc_name):
    import gimpshelf

    (blurb, help, author, copyright, date, label, imagetypes, plugin_type, params, results, function, menu, domain, on_query, on_run) = _registered_plugins_[proc_name]

    key = "python-fu-save--" + proc_name

    if gimpshelf.shelf.has_key(key):
        return gimpshelf.shelf[key]
    else:
        # return the default values
        return [x[3] for x in params]

def _set_defaults(proc_name, defaults):
    import gimpshelf

    key = "python-fu-save--" + proc_name
    gimpshelf.shelf[key] = defaults

def _interact(proc_name, start_params):
    (blurb, help, author, copyright, date, label, imagetypes, plugin_type, params, results, function, menu, domain, on_query, on_run) = _registered_plugins_[proc_name]

    def run_script(run_params):
        params = start_params + tuple(run_params)
        _set_defaults(proc_name, params)
        return apply(function, params)

    params = params[len(start_params):]

    # short circuit for no parameters ...
    if len(params) == 0:
         return run_script([])

    import pygtk
    pygtk.require('2.0')

    import gimpui, gtk
    gimpui.gimp_ui_init ()  # add MrQ	
#    import pango

    defaults = _get_defaults(proc_name)
    defaults = defaults[len(start_params):]

    class EntryValueError(Exception):
        pass

    def warning_dialog(parent, primary, secondary=None):
        dlg = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING, gtk.BUTTONS_CLOSE, primary)
        if secondary:
            dlg.format_secondary_text(secondary)
        dlg.run()
        dlg.destroy()

    def error_dialog(parent, proc_name):
        import sys, traceback

        exc_str = exc_only_str = _("Missing exception information")
        try:
            etype, value, tb = sys.exc_info()
            exc_str = "".join(traceback.format_exception(etype, value, tb))
            exc_only_str = "".join(traceback.format_exception_only(etype, value))
        finally:
            etype = value = tb = None

        title = _("An error occurred running %s") % proc_name
        dlg = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, title)
        dlg.format_secondary_text(exc_only_str)

        alignment = gtk.Alignment(0.0, 0.0, 1.0, 1.0)
        alignment.set_padding(0, 0, 12, 12)
        dlg.vbox.pack_start(alignment)
        alignment.show()

        expander = gtk.Expander(_("_More Information"));
        expander.set_use_underline(True)
        expander.set_spacing(6)
        alignment.add(expander)
        expander.show()

        scrolled = gtk.ScrolledWindow()
        scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled.set_size_request(-1, 200)
        expander.add(scrolled)
        scrolled.show()

        label = gtk.Label(exc_str)
        label.set_alignment(0.0, 0.0)
        label.set_padding(6, 6)
        label.set_selectable(True)
        scrolled.add_with_viewport(label)
        label.show()

        def response(widget, id):
            widget.destroy()

        dlg.connect("response", response)
        dlg.set_resizable(True)
        dlg.show()

    # define a mapping of param types to edit objects ...
    class StringEntry(gtk.Entry):
        def __init__(self, default=""):
            gtk.Entry.__init__(self)
            self.set_text(str(default))
            #self.set_line_wrap(True)
            #self.set_single_line_mode(False)

        def get_value(self):
            return self.get_text()

    class LabelTitle(gtk.Label):
        def __init__(self, default="", align = (0.0, 0.0)):
            gtk.Label.__init__(self)
            self.set_label(str(default))
            self.set_use_markup(True)
            if align != None:
                self.set_alignment(align[0],align[1])
        def get_value(self):
            return self.get_label()

    class TextEntry(gtk.ScrolledWindow):
        def __init__ (self, default=""):
            gtk.ScrolledWindow.__init__(self)
            self.set_shadow_type(gtk.SHADOW_IN)

            self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
            self.set_size_request(100, -1)

            self.view = gtk.TextView()
            self.add(self.view)
            self.view.show()
            self.buffer = self.view.get_buffer()
            self.set_value(str(default))

        def set_value(self, text):
            self.buffer.set_text(text)

        def get_value(self):
            return self.buffer.get_text(self.buffer.get_start_iter(), self.buffer.get_end_iter())

    class IntEntry(StringEntry):
        def get_value(self):
            try:
                return int(self.get_text())
            except ValueError, e:
                raise EntryValueError, e.args

    class FloatEntry(StringEntry):
            def get_value(self):
                try:
                    return float(self.get_text())
                except ValueError, e:
                    raise EntryValueError, e.args

    def precision(step):
        # calculate a reasonable precision from a given step size
        if math.fabs(step) >= 1.0 or step == 0.0:
            digits = 0
        else:
            digits = abs(math.floor(math.log10(math.fabs(step))));
        if digits > 20:
            digits = 20
        return int(digits)

    class SliderEntry(gtk.HScale):
        # bounds is (upper, lower, step)
        def __init__(self, default=0, bounds=(0, 100, 5)):
            step = bounds[2]
            self.adj = gtk.Adjustment(default, bounds[0], bounds[1], step, 10 * step, 0)
            gtk.HScale.__init__(self, self.adj)
            self.set_digits(precision(step))

        def get_value(self):
            return self.adj.value

    class SpinnerEntry(gtk.SpinButton):
        # bounds is (upper, lower, step)
        def __init__(self, default=0, bounds=(0, 100, 5)):
            step = bounds[2]
            self.adj = gtk.Adjustment(default, bounds[0], bounds[1], step, 10 * step, 0)
            gtk.SpinButton.__init__(self, self.adj, step, precision(step))

    class ToggleEntry(gtk.ToggleButton):
        def __init__(self, default=0, labels=[_("No"),_("Yes")]):
            gtk.ToggleButton.__init__(self)
            self.labels = labels
            self.label = gtk.Label(labels[0])
            self.add(self.label)
            self.label.show()
            self.connect("toggled", self.changed)
            self.set_active(default)

        def changed(self, tog):
            if tog.get_active():
                self.label.set_text(self.labels[1])
            else:
                self.label.set_text(self.labels[0])

        def get_value(self):
            return self.get_active()

    class RadioEntry(gtk.VBox):
        def __init__(self, default=0, items=((_("Yes"), 1), (_("No"), 0))):
            gtk.VBox.__init__(self, homogeneous=False, spacing=2)

            button = None

            for (label, value) in items:
                button = gtk.RadioButton(button, label)
                self.pack_start(button)
                button.show()

                button.connect("toggled", self.changed, value)

                if value == default:
                    button.set_active(True)
                    self.active_value = value

        def changed(self, radio, value):
            if radio.get_active():
                self.active_value = value

        def get_value(self):
            return self.active_value

    class ComboEntry(gtk.ComboBox):
        def __init__(self, default=0, items=()):
            store = gtk.ListStore(str)
            for item in items:
                store.append([item])

            gtk.ComboBox.__init__(self, model=store)

            cell = gtk.CellRendererText()
            self.pack_start(cell)
            self.set_attributes(cell, text=0)

            self.set_active(default)

        def get_value(self):
            return self.get_active()

    def FileSelector(default=""):
        # FIXME: should this be os.path.separator?  If not, perhaps explain why?
        if default and default.endswith("/"):
            selector = DirnameSelector
            if default == "/": default = ""
        else:
            selector = FilenameSelector
        return selector(default)

    class FilenameSelector(gtk.FileChooserButton):
        def __init__(self, default="", save_mode=False):
            gtk.FileChooserButton.__init__(self,_("Python-Fu File Selection"))
            self.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
            if default:
                self.set_filename(default)

        def get_value(self):
            return self.get_filename()

    class DirnameSelector(gtk.FileChooserButton):
        def __init__(self, default=""):
            gtk.FileChooserButton.__init__(self,_("Python-Fu Folder Selection"))
            self.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
            if default:
                self.set_filename(default)

        def get_value(self):
            return self.get_filename()

    class DrawCanvas(gtk.DrawingArea):
        def __init__(self,default, size=(100,100)):
            gtk.DrawingArea.__init__(self)
            self.set_size_request(size[0],size[1])
        def get_value(self):
            return 0

    _edit_mapping = {
            PF_INT8        : IntEntry,
            PF_INT16       : IntEntry,
            PF_INT32       : IntEntry,
            PF_FLOAT       : FloatEntry,
            PF_STRING      : StringEntry,
            PF_COLOR       : gimpui.ColorSelector,
            PF_ITEM        : IntEntry,  # should handle differently ...
            PF_IMAGE       : gimpui.ImageSelector,
            PF_LAYER       : gimpui.LayerSelector,
            PF_CHANNEL     : gimpui.ChannelSelector,
            PF_DRAWABLE    : gimpui.DrawableSelector,
            PF_VECTORS     : gimpui.VectorsSelector,

            PF_TOGGLE      : ToggleEntry,
            PF_SLIDER      : SliderEntry,
            PF_SPINNER     : SpinnerEntry,
            PF_RADIO       : RadioEntry,
            PF_OPTION      : ComboEntry,

            PF_FONT        : gimpui.FontSelector,
            PF_FILE        : FileSelector,
            PF_FILENAME    : FilenameSelector,
            PF_DIRNAME     : DirnameSelector,
            PF_BRUSH       : gimpui.BrushSelector,
            PF_PATTERN     : gimpui.PatternSelector,
            PF_GRADIENT    : gimpui.GradientSelector,
            PF_PALETTE     : gimpui.PaletteSelector,
            PF_TEXT        : TextEntry,
            PF_TAB         : TextEntry,
            PF_DRAW        : DrawCanvas,
            PF_TITLE       : LabelTitle,
    }
    # JFGARCIA - modified: to pass args to the on_run
    if on_run:
        on_run(start_params+tuple(params))

    dialog = gimpui.Dialog(proc_name, "python-fu", None, 0, None, proc_name, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))

    dialog.set_alternative_button_order((gtk.RESPONSE_OK, gtk.RESPONSE_CANCEL))

    dialog.set_transient()

    vbox = gtk.VBox(False, 12)
    vbox.set_border_width(12)
    dialog.vbox.pack_start(vbox)
    vbox.show()

    if blurb:
        if domain:
            try:
                (domain, locale_dir) = domain
                trans = gettext.translation(domain, locale_dir, fallback=True)
            except ValueError:
                trans = gettext.translation(domain, fallback=True)
            blurb = trans.ugettext(blurb)
        box = gimpui.HintBox(blurb)
        vbox.pack_start(box, expand=False)
        box.show()

    table = gtk.Table(1, 2, False)
    table.set_row_spacings(2)
    table.set_col_spacings(2)
    vbox.pack_start(table, expand=True)
    table.show()

    notebook = gtk.Notebook()
    tables=[]
    for i in range(len(params)):
        if params[i][0]==PF_TAB:
            vals = params[i][4]
            currPage = params[i][3]

            for n in vals:
                tbl = gtk.Table(2, 2, False)
                tbl.set_row_spacings(5)
                tbl.set_col_spacings(5)
                tbl.show()
                tables.append({'table':tbl,'rows':0, 'totCols':0})
                notebook.append_page(tbl, gtk.Label(n))
                notebook.set_current_page(currPage)


    if len(tables)>0:
        table.attach(notebook, 0,6,0,1)
        notebook.show()
    else:
        tables.append({'table':table,'rows':0})

    def response(dlg, id):
        if id == gtk.RESPONSE_OK:
            dlg.set_response_sensitive(gtk.RESPONSE_OK, False)
            dlg.set_response_sensitive(gtk.RESPONSE_CANCEL, False)
            params = []
            try:
                for wid in edit_wids:
                    params.append(wid.get_value())
            except EntryValueError:
                warning_dialog(dialog, _("Invalid input for '%s'") % wid.desc)
            else:
                try:
                    dialog.res = run_script(params)
                except Exception:
                    dlg.set_response_sensitive(gtk.RESPONSE_CANCEL, True)
                    error_dialog(dialog, proc_name)
                    raise
        gtk.main_quit()

    iniX=0
    dialog.connect("response", response)
    row=0
    edit_wids = []

    for i in range(len(params)):
        paramsi = params[i]
        pf_type = paramsi[0]
        name, desc = (paramsi[1], paramsi[2])
        def_val = defaults[i]
 
        tab, incr, rows, connect, cols, hlp = (0, 1, 1, None, 1, desc)
        if len(params[i])>5:
            xtra = params[i][5]
            incr = int(getXtra(xtra, 'newLine', 1))
            if incr==0 : cols = 1
            cols = getXtra(xtra, 'cols'   , cols)
            iniX = getXtra(xtra, 'iniCol' , iniX)
            rows = getXtra(xtra, 'rows'   , 1)
            tab  = getXtra(xtra, 'tab'    , 0)
            hlp  = getXtra(xtra, 'help'   , desc)
            connect = getXtra(xtra,'connect', None)
        else:
            iniX = 0
        tbl = tables[tab]['table']
        row = tables[tab]['rows']


        if pf_type==PF_TAB:
            wid = _edit_mapping[pf_type](None)
        else:
            if pf_type != PF_TITLE and desc != '':
                label = gtk.Label(desc)
                label.set_use_underline(True)
                label.set_alignment(0.0, 0.5)
                tbl.attach(label, iniX + 1, iniX + 2, row, row + rows, xoptions = gtk.FILL)
                label.show()

            if pf_type in (PF_SPINNER, PF_SLIDER, PF_RADIO, PF_OPTION, PF_DRAW ):
                wid = _edit_mapping[pf_type](def_val, paramsi[4])
            elif pf_type==PF_TOGGLE:
                if isinstance(paramsi[4], list):
                    wid = _edit_mapping[pf_type](def_val, paramsi[4])
                else:
                    wid = _edit_mapping[pf_type](def_val)
            elif pf_type==PF_TITLE:
                wid = _edit_mapping[pf_type](desc)
            else:
                wid = _edit_mapping[pf_type](def_val)
            if pf_type!=PF_TITLE and desc != '':
                label.set_mnemonic_widget(wid)
                tbl.attach(wid, iniX + 2, iniX + 2 + cols, row, row + rows, yoptions=0)
                iniX = iniX + cols + 1
            else:
                tbl.attach(wid, iniX, iniX+cols, row, row + rows, xoptions=gtk.FILL)
            # jfGarcia: strip html tags from toolTip
            tTip = re.sub('<[^<]+?>', '', hlp)
            if pf_type != PF_TEXT:
                wid.set_tooltip_text(tTip)
            else:
                # Attach tip to TextView, not to ScrolledWindow
                wid.view.set_tooltip_text(tTip)

            wid.set_name(name)
            if connect:
                try:
                    wid.connect(connect['signal'],connect['function'])
                except Exception, e:
                    pass
            wid.show()
            wid.desc = desc

        edit_wids.append(wid)

        tables[tab]['rows'] += incr
        if incr==1:
           iniX=0

    progress_vbox = gtk.VBox(False, 6)
    vbox.pack_end(progress_vbox, expand=False)
    progress_vbox.show()

    progress = gimpui.ProgressBar()
    progress_vbox.pack_start(progress)
    progress.show()

    dialog.show()

    gtk.main()

    if hasattr(dialog, "res"):
        res = dialog.res
        dialog.destroy()
        return res
    else:
        dialog.destroy()
        raise CancelError

def _run(proc_name, params):
    run_mode = params[0]
    func = _registered_plugins_[proc_name][10]

    if run_mode == RUN_NONINTERACTIVE:
        return apply(func, params[1:])

    script_params = _registered_plugins_[proc_name][8]

    min_args = 0
    if len(params) > 1:
        for i in range(1, len(params)):
            param_type = _obj_mapping[script_params[i - 1][0]]
            if not isinstance(params[i], param_type):
                break
        min_args = i
    if len(script_params) > min_args:
        start_params = params[:min_args + 1]
        if run_mode == RUN_WITH_LAST_VALS:
            default_params = _get_defaults(proc_name)
            params = start_params + default_params[min_args:]
        else:
            params = start_params
    else:
       run_mode = RUN_NONINTERACTIVE
    if run_mode == RUN_INTERACTIVE:
        try:
            res = _interact(proc_name, params[1:])
        except CancelError:
            return
    else:
        res = apply(func, params[1:])
        gimp.displays_flush()
    return res

def main():
    """This should be called after registering the plug-in."""
    gimp.main(None, None, _query, _run)

def fail(msg):
    """Display an error message and quit"""
    gimp.message(msg)
    raise error, msg

def N_(message):
    return message


############################################################################################################################################################
######################################################### END gimpfu3 ######################################################################################
############################################################################################################################################################

import os
import random

# the color parameter in Gmic filters requires only three parameters (r, g, b) - Gimp takes four parameters (r, g, b, a)
def gmic_rgba_to_rgb(rgba_color):

	r = rgba_color[0]
	g = rgba_color[1]
	b = rgba_color[2]

	# write rgb in correct syntax
	rgb_color = "{0},{1},{2}".format(r,g,b)
	return rgb_color
	
def gmic_hex_to_rgb(hex_color):
    hex_color = hex_color.replace(" ", "").replace("#", "")
    # 3-digits hex color
    if len(hex_color) == 3:
        r = hex_color[0] * 2
        g = hex_color[1] * 2
        b = hex_color[2] * 2
    # 6-digits hex color
    elif len(hex_color) == 6:
        r = hex_color[0:2]
        g = hex_color[2:4]
        b = hex_color[4:6]
    else:
        return "length error"
    # convert hex to decimal
    r = int(r, 16)
    g = int(g, 16)
    b = int(b, 16)

    # check if in range 0~255
    assert 0 <= r <= 255
    assert 0 <= g <= 255
    assert 0 <= b <= 255
    # write rgb in correct syntax
    rgb_color = "{0},{1},{2}".format(r, g, b)
    return rgb_color
gmic_command=[ 		 	             
	" RANDOM ",
	" 1. Angoisse Anguish ",
	" 2. Aurora ",
	" 3. Barbouillage Paint Daub ",
	" 4. Black Crayon Graffiti ",
	" 5. Blockism ",
	" 6. Bokeh ",
	" 7. Brushify ",
 	" 8. Canvas ",
	" 9. Canvas Texture ",
	"10. Clouds ",
	"11. Chalk It Up ",
	"12. Color Abstraction Paint ",
	"13. Colored Engraving ", 
	"14. Colored Outlines ", 
	"15. Colored Pencil ",
	"16. Colorize [With Colormap] ",
	"17. Comic Book ",
	"18. Contours Arrondis ",
	"19. Contours Colores ",
	"20. Contours Gros Pixels ",
	"21. Couleurs Metalliques ",
	"22. Couleurs Rayees ",	
	"23. Cracks ",
	"24. Cubism ",
	"25. Diffusion Tensors ",
	"26. Doodle ",
	"27. Dorure Gilding ",			
	"28. Dream Smoothing ",
	"29. Edges on Fire ",
	"30. Engrave ",
	"31. Fractalize ",
	"32. Fx Sketchbw Modifie ",	
	"33. Ghost ",
	"34. Gradient Norm ",
	"35. Gradient RGB ",
	"36. Granular Texture ",		
	"37. Graphic Boost ",
	"38. Graphic Novel ",
	"39. Grid [ triangular ] ",
	"40. Halftone ",
	"41. Hallucinogen ",
	"42. Hard Sketch ",
	"43. Hough Sketch ",
	"44. Impressions ",
	"45. Isophotes Vers Aquarelle ",
	"46. Lava ",
	"47. Light Patch ",
	"48. Lineart ",
	"49. Lylejk's Stencil ",
	"50. Marble ",
	"51. Metallic Stencils ",
	"52. Mineral Mosaic ",
	"53. Morphological ",
	"54. Pack Ellipses ",	
	"55. Paint Daub ",
	"56. Paper Texture ",
	"57. Pen Drawing ",
	"58. Plasmicv2 ",
	"59. Polygonize [ Energy ] ",
	"60. Random Plasma ",
	"61. Relief Light ",
	"62. Ripple ",
	"63. Rodilius ",
	"64. Shock Waves ",
	"65. Sketch ",	
	"66. Sloppy Mess ",
	"67. Sponge ",
	"68. Stained Glass ",
	"69. Stencil ",
	"70. Strip ",
	"71. Tetris ",
	"72. Textured Glass ",	
	"73. Turbulent Halftone ",
	"74. Voronoi ",
	"75. Warp by Intensity ",
	"76. Weave ",
	"77. Whirl Drawing ",
	"78. Whirls ",
	"79. Wind ",
	"80. Woven [Lylejk] ",
	]	
def python_fu_gmic_exercise(image, layer, options, text, margin, font, direction, just, letter_spacing, line_spacing, font_size,fill_t,text_color,text_pattern,fill_b,background_color,b_pattern,background_opacity, gmic1,gmic2, frame, frame_color, flatten,ptab):
	
	frame_command =[
	" RANDOM ", 
	" DON'T USE ",
	" Blur ",
	" Fuzzy ",
	" Mirror ",
	" Painting ", 
	" Painting At06A_2017 ",
	" Regular ",
	" Relief ",
	" Round ",
	" Old Photograph ",
	" Polaroid ",
	" Vignette "]
	
	if options==0:	# Text Logo
		
		image = gimp.Image(int(font_size), int(font_size), RGB)

		# create layer with text and add it to the image: image, drawable, x, y, text, border, antialias, size, size_type, fontname
		text_layer = pdb.gimp_text_fontname(image, None, 0, 0, text, 0, 1, font_size, PIXELS, font)

		# justification:
		pdb.gimp_text_layer_set_base_direction (text_layer, direction)
		pdb.gimp_text_layer_set_justification (text_layer, just)
		pdb.gimp_text_layer_set_letter_spacing(text_layer, letter_spacing)
		pdb.gimp_text_layer_set_line_spacing(text_layer, line_spacing)
		
		if fill_t==0:
			 pdb.gimp_text_layer_set_color(text_layer, text_color)
			 
		pdb.gimp_image_set_active_layer(image,text_layer)		
		pdb.plug_in_autocrop_layer(image, text_layer)
		pdb.gimp_image_resize_to_layers(image)
		
		if fill_t==1:
			pdb.gimp_image_select_item(image,0,text_layer)
			pdb.gimp_context_set_pattern(text_pattern)
			pdb.gimp_edit_fill(text_layer,4)
			pdb.gimp_selection_none(image)
			
		wi = pdb.gimp_image_width(image)
		hi = pdb.gimp_image_height(image)
		
		new_width = wi + 2*margin
		new_height = hi + 2*margin
		
		wh = pdb.gimp_drawable_width(text_layer)
		hh = pdb.gimp_drawable_height(text_layer)
		centerx=wi/2
		centery=hi/2
		adw=wh/2
		adh=hh/2
		sw=centerx - adw
		sh=centery - adh		
		pdb.gimp_layer_set_offsets(text_layer, sw, sh)
		pdb.gimp_image_resize(image,new_width,new_height,margin, margin)
		pdb.gimp_layer_resize_to_image_size(text_layer)
			
		# create layer: image, width, height, type, name, opacity, mode
		layer = gimp.Layer(image, "background", image.width, image.height, RGBA_IMAGE, background_opacity, 0)

		# append layer to the image
		image.add_layer(layer, 1)

		# fill layer with color
	
		if fill_b==0:
			gimp.set_background(background_color)
			pdb.gimp_edit_fill(layer,1)
			layer = image.merge_visible_layers(CLIP_TO_IMAGE)
		if fill_b==1:
			pdb.gimp_image_select_item(image,0,layer)
			pdb.gimp_context_set_pattern(b_pattern)
			pdb.gimp_edit_fill(layer,4) 		
			#layer.fill(FILL_BACKGROUND)
			layer = image.merge_visible_layers(CLIP_TO_IMAGE)	
			
		
	if options==1:	# Image	
		active_layer = pdb.gimp_image_get_active_layer(image)
		w = pdb.gimp_drawable_width(active_layer)
		h = pdb.gimp_drawable_height(active_layer)

		image = pdb.gimp_image_new(w, h, 0)

		display = pdb.gimp_display_new(image)
		#pdb.gimp_image_undo_group_start(new_image)
		active_layer = pdb.gimp_layer_new_from_drawable(layer, image)
		pdb.gimp_image_insert_layer(image, active_layer, None, 0)	
		active_layer.name = layer.name
		layer = active_layer
		pdb.gimp_displays_flush()
		#pdb.gimp_image_select_item(image, 2, layer)

#pdb.plug_in_gmic_qt(image, drawable, input, output, command)
#Input layers mode, when non-interactive (0=none, 1=active, 2=all, 3=active & below, 4=active & above,5=all visibles, 6=all invisibles, 7=all visibles (decr.), 8=all invisibles (decr.), 9=all (decr.))
#Output mode, when non-interactive (0=in place,1=new layers,2=new active layers,3=new image)
	if gmic1==0:	# Logo Image Random
		gmic1 = random.randrange(1,51)

		
	if gmic2==0:	# Logo Image Random
		gmic2 = random.randrange(1,51)
		
	if frame==0:		
		frame = random.randrange(2,12)
		#print pdb.gimp_message("C1=" + str(gmic1) + " " + "C2=" + str(gmic2))		

	if gmic1==1:	# Angoisse Anguish
		x='samj_Angoisse 1,5,0,5,100,2,4,1,250'
	if gmic2==1:		
		y='samj_Angoisse 1,5,0,5,100,2,4,1,250'
		
	if gmic1==2:	# Aurora
		x='gcd_aurora 6,1,0 '
	if gmic2==2:		
		y='gcd_aurora 6,1,0 '
		
	if gmic1==3:	# Barbouillage Paint Daub
		x='samj_Barbouillage_Paint_Daub 2,2,100,0.2,1,4,1,0,8'
	if gmic2==3:		
		y='samj_Barbouillage_Paint_Daub 2,2,100,0.2,1,4,1,0,8'
		
	if gmic1==4:	# Black Crayon Graffiti
		x='fx_crayongraffiti2 300,50,1,0.4,12,1,2,2,0'
	if gmic2==4:		
		y='fx_crayongraffiti2 300,50,1,0.4,12,1,2,2,0'
		
	if gmic1==5:	# Blockism
		x='fx_blockism 3,1.6,0.5,50,0.5,64,0,1,0'
	if gmic2==5:		
		y='fx_blockism 3,1.6,0.5,50,0.5,64,0,1,0'
		
	if gmic1==6:	# Bokeh
		x='fx_bokeh 3,8,0,30,8,4,0.3,0.2,210,210,80,160,0.7,30,20,20,1,2,170,130,20,110,0.15,0,50,50'
	if gmic2==6:		
		y='fx_bokeh 3,8,0,30,8,4,0.3,0.2,210,210,80,160,0.7,30,20,20,1,2,170,130,20,110,0.15,0,50,50'
		
	if gmic1==7:	# Brushify
		x='fx_brushify 7,0.25,4,64,25,12,0,2,4,0.2,0.5,30,1,1,1,5,0,0.2,1'
	if gmic2==7:		
		y='fx_brushify 7,0.25,4,64,25,12,0,2,4,0.2,0.5,30,1,1,1,5,0,0.2,1'		

	if gmic1==8:	# 1 canvas
		x='fx_canvas 70,45,400,1,70,135,400,0,50,50'
	if gmic2==8:		
		y='fx_canvas 70,45,400,1,70,135,400,0,50,50'	

	if gmic1==9:	# 2 canvas texture
		x='texturize_canvas 20,3,0.6'
	if gmic2==9:		
		y='texturize_canvas 20,3,0.6'			

	if gmic1==10:	# 10 Clouds
		x='-jeje_clouds 70,0.5,50,50'
	if gmic2==10:		
		y='-jeje_clouds 70,0.5,50,50'	

		  
	if gmic1==11:	# 11 Chalk It Up
		x='samj_chalkitup 5,2.5,1.5,50,1,5,5,0,0,7,0.8,1.9,7,0'
	if gmic2==11:		
		y='samj_chalkitup 5,2.5,1.5,50,1,5,5,0,0,7,0.8,1.9,7,0'
		
	if gmic1==12:	# 12 Color Abstraction Paint
		x='fx_ColorAbstractionPaint 5,10,1,0,1,0,1,0,0,0,0,0,0,0'
	if gmic2==12:		
		y='fx_ColorAbstractionPaint 5,10,1,0,1,0,1,0,0,0,0,0,0,0'		
		
	if gmic1==13:	# 13 Colored Engraving
		x='Engrave_colore 0,0.5,4,0,8,40,0,25,1,2,0,0,0,255,255,255'
	if gmic2==13:		
		y='Engrave_colore 0,0.5,4,0,8,40,0,25,1,2,0,0,0,255,255,255'
		
	if gmic1==14:	# 14 Colored Outlines
		x='samj_Colored_Outlines 0,2,8,0,0,0,0,0,255'
	if gmic2==14:		
		y='samj_Colored_Outlines 0,2,8,0,0,0,0,0,255'

	if gmic1==15:	# 15 Colored Pencil
		x='fx_cpencil 1.3,50,20,2,2,1,0'
	if gmic2==15:		
		y='fx_cpencil 1.3,50,20,2,2,1,0'		
		
	if gmic1==16:	# 16 Colorize [With Colormap]
		x='fx_bwrecolorize 0,0,0,0,0,1,0,5,0,0,0,255,43,25,55,255,158,137,189,255,224,191,228,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,50,50'
	if gmic2==16:		
		y='fx_bwrecolorize 0,0,0,0,0,1,0,5,0,0,0,255,43,25,55,255,158,137,189,255,224,191,228,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,50,50'
		
	if gmic1==17:	# 17 Comic Book
		x='cl_comic 0,2,0,1,1,15,15,1,10,20,6,2,0,0,0,0,0,0,50,50'
	if gmic2==17:		
		y='cl_comic 0,2,0,1,1,15,15,1,10,20,6,2,0,0,0,0,0,0,50,50'		
				
	if gmic1==18:	# 6 Contours Arrondis
		x='samj_Contours_Arrondis 1,3,5,10,0,0'
	if gmic2==18:		
		y='samj_Contours_Arrondis 1,3,5,10,0,0'
		
	if gmic1==19:	# 7 Contours Colores
		x='samj_Contours_Colores 1.1,2,40,0,0'
	if gmic2==19:		
		y='samj_Contours_Colores 1.1,2,40,0,0'
		
	if gmic1==20:	# 8 Contours Gros Pixels
		x='samj_Contours_Gros_Pixels 1,8,0,20,20,16,1.1,0,0,15,0,0,0,1'
	if gmic2==20:		
		y='samj_Contours_Gros_Pixels 1,8,0,20,20,16,1.1,0,0,15,0,0,0,1'
		
		
		
	if gmic1==21:	# 9 Couleurs Metalliques
		x='-Couleurs_Metalliques 0,0,22,0,0,0,0,6,5,20,1,0,0,0,0.3,60,1,1,0'
	if gmic2==21:		
		y='-Couleurs_Metalliques 0,0,22,0,0,0,0,6,5,20,1,0,0,0,0.3,60,1,1,0'

	if gmic1==22:	# 10 Couleurs Rayees
		x='samj_Couleurs_Rayees 0,30,0,100,4,4,0,2,0.7,0'
	if gmic2==22:		
		y='samj_Couleurs_Rayees 0,30,0,100,4,4,0,2,0.7,0'
	
	if gmic1==23:	# 11. Cracks
		x='-fx_cracks 30,1,255,255,255,128,0,0,50,50'
	if gmic2==23:		
		y='-fx_cracks 30,1,255,255,255,128,0,0,50,50'
		
	if gmic1==24:	# cubism
		x='fx_cubism 2,50,10,90,0.7,0,0,50,50'
	if gmic2==24:		
		y='fx_cubism 2,50,10,90,0.7,0,0,50,50'		

	if gmic1==25:	# 12 Diffusion tensors
		x='-fx_diffusiontensors 10,5,2,1,0.15,1,0,3,0'
	if gmic2==25:		
		y='-fx_diffusiontensors 10,5,2,1,0.15,1,0,3,0'

	if gmic1==26:	# 13 Doodle
		x='-fx_doodle 30,2,2,1.5,2,70,1'
	if gmic2==26:		
		y='-fx_doodle 30,2,2,1.5,2,70,1'
		
	if gmic1==27:	# 14 Dorure Gilding
		x='samj_Dorure 0.4,0.2,0.2,0,1,20,20,0,0.5,0,60,0,0,100,45,18,32,1'
	if gmic2==27:		
		y='samj_Dorure 0.4,0.2,0.2,0,1,20,20,0,0.5,0,60,0,0,100,45,18,32,1'
		
	if gmic1==28:	# 15 Dream Smoothing
		x='-fx_dreamsmooth 3,1,1,0.8,0,0.8,1,24,0'
	if gmic2==28:		
		y='-fx_dreamsmooth 3,1,1,0.8,0,0.8,1,24,0'		
		
	if gmic1==29:	# 17. Edges on Fire
		x='-fx_fire_edges 0.7,0.25,0.5,25,1,20,0,0,50,50'
	if gmic2==29:		
		y='-fx_fire_edges 0.7,0.25,0.5,25,1,20,0,0,50,50'
		
	if gmic1==30:	# 18. Engrave
		x='-fx_engrave 0.5,50,0,8,40,0,0,0,10,1,0,0,0,1,0'
	if gmic2==30:		
		y='-fx_engrave 0.5,50,0,8,40,0,0,0,10,1,0,0,0,1,0'
		

	if gmic1==31:	# fractalize
		x='fractalize 0.8'
	if gmic2==31:		
		y='fractalize 0.8'
		
	if gmic1==32:	# Fx Sketchbw Modifie
		x='samj_fx_sketchbw_modifie 3,45,180,30,1.75,0.02,0.5,0.75,0.1,0.7,3,6,0,1,4,0,1234,0,2,100,0.2,1,4,1,0'
	if gmic2==32:		
		y='samj_fx_sketchbw_modifie 3,45,180,30,1.75,0.02,0.5,0.75,0.1,0.7,3,6,0,1,4,0,1234,0,2,100,0.2,1,4,1,0'
		
	if gmic1==33:	# ghost
		x='fx_ghost 200,2,2,1,3,16,0,0,50,50'
	if gmic2==33:		
		y='fx_ghost 200,2,2,1,3,16,0,0,50,50'		
	
	if gmic1==34:	# 20. Gradient norm
		x='-fx_gradient_norm 0,0.5,0,100,0,0'
	if gmic2==34:		
		y='-fx_gradient_norm 0,0.5,0,100,0,0'		

	if gmic1==35:	# 21. Gradient RGB
		x='-fx_gradient2rgb 0,0,100,0,0,0'
	if gmic2==35:		
		y='-fx_gradient2rgb 0,0,100,0,0,0'	
	
	if gmic1==36:	#  22 Granular Texture
		x='samj_Texture_Granuleuse 0.5,20,80,0,0,0'
	if gmic2==36:		
		y='samj_Texture_Granuleuse 0.5,20,80,0,0,0'		

	if gmic1==37:	# 23. Graphic boost 
		x='-fx_graphic_boost4 1.25,2,0,0.15,14,0,1,0.5,0.45,2,0,0,0,1,1,1,0.5,0.45,1'
	if gmic2==37:		
		y='-fx_graphic_boost4 1.25,2,0,0.15,14,0,1,0.5,0.45,2,0,0,0,1,1,1,0.5,0.45,1'

	if gmic1==38:	# Graphic Novel
		x='fx_graphic_novelfxl 0,2,6,5,20,0,0.62,14,0,1,0.5,0.78,1.92,0,0,0,1,1,1,0.5,0.8,1.28'
	if gmic2==38:		
		y='fx_graphic_novelfxl 0,2,6,5,20,0,0.62,14,0,1,0.5,0.78,1.92,0,0,0,1,1,1,0.5,0.8,1.28'
		
	if gmic1==39:	# 24 Grid [triangular]
		x='-fx_imagegrid_triangular 8,10,0,255,255,255,128'
	if gmic2==39:		
		y=	'-fx_imagegrid_triangular 8,10,0,255,255,255,128'
		
	if gmic1==40:	# fx_halftone
		x='fx_halftone 0,0,0,0,5,8,8,5,0.1,0,50,50'
	if gmic2==40:		
		y='fx_halftone 0,0,0,0,5,8,8,5,0.1,0,50,50'


		
	if gmic1==41:	# 25 Hallucinogen
		x='samj_Hallucinogen 80,2,80,1.5,2,0,0,0,1,80,8'
	if gmic2==41:		
		y='samj_Hallucinogen 80,2,80,1.5,2,0,0,0,1,80,8'			
		
	if gmic1==42:	# 26 Hard Sketch
		x='fx_hardsketchbw 300,50,1,0.1,20,0,4,0,50,50'
	if gmic2==42:		
		y='fx_hardsketchbw 300,50,1,0.1,20,0,4,0,50,50'
		
	if gmic1==43:	# 27 Hough Sketch
		x='-fx_houghsketchbw 1.25,10,5,80,0.1,4,0,50,50'
	if gmic2==43:		
		y=	'-fx_houghsketchbw 1.25,10,5,80,0.1,4,0,50,50'
		
	if gmic1==44:	# 28 Impressions
		x='samj_Impressions 10,5,5,1,1,0,2,0,1,5,0,5,100,2,4,1,250,1.2'
	if gmic2==44:		
		y='samj_Impressions 10,5,5,1,1,0,2,0,1,5,0,5,100,2,4,1,250,1.2'	
		
	if gmic1==45:	# 29 Isophotes Vers Aquarelle
		x='samj_Isophotes_Vers_Aquarelle 16,0.5,4,0,1,10,1,0,0,0,0,1,10,10,4,3,251,237,206,1'
	if gmic2==45:		
		y='samj_Isophotes_Vers_Aquarelle 16,0.5,4,0,1,10,1,0,0,0,0,1,10,10,4,3,251,237,206,1'	
	
	if gmic1==46:	# 30 Lava
		x='-fx_lava 3,50,0.4,0,0'
	if gmic2==46:		
		y='-fx_lava 3,50,0.4,0,0'	
		
	if gmic1==47:	# 31 Light patch
		x='-fx_light_patch 5,0.7,2.5,0'
	if gmic2==47:		
		y='-fx_light_patch 5,0.7,2.5,0'	
		
	if gmic1==48:	# 48 lineart
		x='cl_lineart 0,0,2,1,15,15,1,0,6,2,2,0,0,0,50,50'
	if gmic2==48:		
		y='cl_lineart 0,0,2,1,15,15,1,0,6,2,2,0,0,0,50,50'	
		
	if gmic1==49:	# 32 Lylejk's stencil
		x='-fx_lylejk_stencil 5,10,3,0'
	if gmic2==49:		
		y='-fx_lylejk_stencil 5,10,3,0'	
		
	if gmic1==50:	# 33 Marble
		x='-fx_marble 0.5,0.3,0,0,0.4,0.6,0.6,1.1,0,100'
	if gmic2==50:		
		y='-fx_marble 0.5,0.3,0,0,0.4,0.6,0.6,1.1,0,100'
		
		
	if gmic1==51:	# 34 Metallic stencils
		x='fx_metallicstencils 0,0'
	if gmic2==51:		
		y='fx_metallicstencils 0,0'
	
	if gmic1==52:	# 51 mineral_mosaic
		x='fx_mineral_mosaic 1,2,1,100,0'
	if gmic2==52:		
		y='fx_mineral_mosaic 1,2,1,100,0'
		
	if gmic1==53:	# 52 Morphological
		x='-fx_morphological 0,0,5,"1,0,1; 0,1,0; 1,0,1",0,0,0,0,0'
	if gmic2==53:		
		y='-fx_morphological 0,0,5,"1,0,1; 0,1,0; 1,0,1",0,0,0,0,0'

	if gmic1==54:	# 54 Pack Ellipses
		x='fx_pack_ellipses 3,20,0,30,100,6,1,3,0,0,0,255,0,0'
	if gmic2==54:		
		y='fx_pack_ellipses 3,20,0,30,100,6,1,3,0,0,0,255,0,0'
				
	if gmic1==55:	# 55 Paint daub
		x='samj_Barbouillage_Paint_Daub 2,2,100,0.2,1,4,1,0,8'
	if gmic2==55:		
		y='samj_Barbouillage_Paint_Daub 2,2,100,0.2,1,4,1,0,8'
		
	if gmic1==56:	# 54 Paper Texture",
		x='fx_paper 0,0,50,50'
	if gmic2==56:		
		y='fx_paper 0,0,50,50'
		
	if gmic1==57:	# 53 Pen drawing",
		x='-fx_pen_drawing 10,0'
	if gmic2==57:		
		y='-fx_pen_drawing 10,0'
		
	if gmic1==58:	# "56 Plasmicv2",
		x='-samj_Plasmic_V2 2,4,10,2,0.02,8,0,0,0,0,0,0,255'
	if gmic2==58:		
		y='-samj_Plasmic_V2 2,4,10,2,0.02,8,0,0,0,0,0,0,255'
		
	if gmic1==59:	# "57 Polygonize [Energy]",
		x='-fx_polygonize 300,10,10,10,10,0,0,0,255,0,50,50'
	if gmic2==59:		
		y='-fx_polygonize 300,10,10,10,10,0,0,0,255,0,50,50'
		
	if gmic1==60:	#  58 Random Plasma
		x='samj_Random_Plasma 0,80'
	if gmic2==60:		
		y='samj_Random_Plasma 0,80'	
		
		
	if gmic1==61:	# 59 Relief light
		x='-fx_light_relief 0.3,0.2,0.2,0,1,0.5,0.5,5,0.5,0,0'
	if gmic2==61:		
		y='-fx_light_relief 0.3,0.2,0.2,0,1,0.5,0.5,5,0.5,0,0'
		
	if gmic1==62:	# 60 Ripple
		x='-ripple 10,20,2,0,0'
	if gmic2==62:		
		y='-ripple 10,20,2,0,0'
		
	if gmic1==63:	# 61 Rodilius",
		x='-fx_rodilius 8.2,4,300,5,30,0,1,0,0,0'
	if gmic2==63:		
		y='-fx_rodilius 8.2,4,300,5,30,0,1,0,0,0'
# 
	if gmic1==64:	# 64 shockwaves",
		x='fx_shockwaves 10,10,20,0,0,50,50'
	if gmic2==64:		
		y='fx_shockwaves 10,10,20,0,0,50,50'
		
	if gmic1==65:	# 65 Sketch
		x='-fx_sketchbw 3,45,180,30,1.75,0.02,0.5,0.75,0.1,0.7,3,6,0,1,4,0,50,50'
	if gmic2==65:		
		y='-fx_sketchbw 3,45,180,30,1.75,0.02,0.5,0.75,0.1,0.7,3,6,0,1,4,0,50,50'
		
	if gmic1==66:	# 66 Sloppy Mess
		x='fx_sloppymess 0,1,0,0,-1,0,35,-35,2,0,0,3,1,3,100'
	if gmic2==66:		
		y='fx_sloppymess 0,1,0,0,-1,0,35,-35,2,0,0,3,1,3,100'			
		
	if gmic1==67:	# 64 Sponge
		x='fx_sponge 13,0,0,50,50'
	if gmic2==67:		
		y='fx_sponge 13,0,0,50,50'		
# 		
	if gmic1==68:	# 65 Stained glass
		x='-fx_stained_glass 20,0.1,1,1,1,0,0,0,0'
	if gmic2==68:		
		y='-fx_stained_glass 20,0.1,1,1,1,0,0,0,0'
		
	if gmic1==69:	# 66 Stencil",
		x='-fx_stencil 3,0,8,0,2,0,0'
	if gmic2==69:		
		y='-fx_stencil 3,0,8,0,2,0,0'
		
	if gmic1==70:	# 67 Strip"
		x='-jeje_strip 45,50,0,1,0'
	if gmic2==70:		
		y='-jeje_strip 45,50,0,1,0'
		
		
	if gmic1==71:	# 68 Tetris"
		x='fx_tetris 10'
	if gmic2==71:		
		y='fx_tetris 10'		

	if gmic1==72:	# 72. Textured Glass"
		x='fx_textured_glass 40,40,1,1,0,10,4,0,50,50'
	if gmic2==72:		
		y='fx_textured_glass 40,40,1,1,0,10,4,0,50,50'

	if gmic1==73:	# 73 Turbulent Halftone"
		x='iain_turbulent_halftone 15,20,0,1,512,0.75,0,0,0'
	if gmic2==73:		
		y='iain_turbulent_halftone 15,20,0,1,512,0.75,0,0,0'

	if gmic1==74:	# 74 Voronoi"
		x='fx_voronoi 160,1,0.5,50,3,1,0,0,0,100,2,255,255,255,40,1'
	if gmic2==74:		
		y='fx_voronoi 160,1,0.5,50,3,1,0,0,0,100,2,255,255,255,40,1'
		
	if gmic1==75:  	#  75 Warp by intensity
		x='fx_warp_by_intensity 0.04,0.04,128,128,0,1,3,0,0,50,50'
	if gmic2==75:		
		y='fx_warp_by_intensity 0.04,0.04,128,128,0,1,3,0,0,50,50'	
		
	if gmic1==76:	# " 76 Weave ",
		x='weave 6,65,0,0.5,0,0,0,0,0'
	if gmic2==76:		
		y='weave 6,65,0,0.5,0,0,0,0,0'		

	if gmic1==77:	# "77. Whirl Drawing",
		x='fx_draw_whirl 20,0,50,50'
	if gmic2==77:		
		y='fx_draw_whirl 20,0,50,50'

	if gmic1==78:	# "76 Whirls",
		x='-fx_whirls 7,2,0.2,1.8,11,0'
	if gmic2==78:		
		y='-fx_whirls 7,2,0.2,1.8,11,0'
		
	if gmic1==79:	# 78. Wind",
		x='-fx_wind 20,0,0.7,20,1,0,0,0,50,50'
	if gmic2==79:		
		y='-fx_wind 20,0,0.7,20,1,0,0,0,50,50'
		
	if gmic1==80:	# 80. Woven [Lylejk]",
		x='Lylejk_Woven 19.6,32.4,1,0,0,19.6,32.4,1,0,0,12,10,5,1,1,0,0,0,255,19.6,32.4,1,0,0,19.6,32.4,1,0,0,150,0.61,0.85,0.6,7.83,0.68,19,2.64,0,1,1,0,0.29,37.2,100,0,0,2,0,16,0'
	if gmic2==80:		
		y='Lylejk_Woven 19.6,32.4,1,0,0,19.6,32.4,1,0,0,12,10,5,1,1,0,0,0,255,19.6,32.4,1,0,0,19.6,32.4,1,0,0,150,0.61,0.85,0.6,7.83,0.68,19,2.64,0,1,1,0,0.29,37.2,100,0,0,2,0,16,0'

#pdb.plug_in_gmic_qt(image, drawable, input, output, command)
#Input layers mode, when non-interactive (0=none, 1=active, 2=all, 3=active & below, 4=active & above,5=all visibles, 6=all invisibles, 7=all visibles (decr.), 8=all invisibles (decr.), 9=all (decr.))
#Output mode, when non-interactive (0=in place,1=new layers,2=new active layers,3=new image)

	gimp.pdb.plug_in_gmic_qt( image, layer, 1, 2, x, run_mode=RUN_NONINTERACTIVE)
	active_layer_1 = pdb.gimp_image_get_active_layer(image)
	active_layer_1.name = str(gmic_command[gmic1])
	gimp.pdb.plug_in_gmic_qt( image, layer, 1, 2, y, run_mode=RUN_NONINTERACTIVE)
	active_layer_2 = pdb.gimp_image_get_active_layer(image)	
	active_layer_2.name = str(gmic_command[gmic2])

	#hex = gmic_hex_to_rgb(frame_color)	
	rgb = gmic_rgba_to_rgb(frame_color)	
	
	if frame==2:	# " Frame [Blur]
		gimp.pdb.plug_in_gmic_qt( image, layer, 1, 2, '-fx_frame_blur 30,30,0,5,0,0,128,128,128,0,1,' + str(rgb) + ',2,2,1,0,0.50,0.50,0.0',run_mode=RUN_NONINTERACTIVE)
	if frame==3:	# Frame [Fuzzy]
		gimp.pdb.plug_in_gmic_qt( image, layer, 1, 2, 'fx_frame_fuzzy 5,5,10,1,' + str(rgb) + ',255',run_mode=RUN_NONINTERACTIVE)
	if frame==4:	# " " Frame [Mirror]
		gimp.pdb.plug_in_gmic_qt( image, layer, 1, 2, 'fx_frame_mirror 10,10,50,50,0,0,0,0,0.75',run_mode=RUN_NONINTERACTIVE)
	if frame==5:	# " Frame [Painting] "
		gimp.pdb.plug_in_gmic_qt( image, layer, 1, 2, '	fx_frame_painting 10,0.4,6,' + str(rgb) + ',2,400,50,10,1,0.5,123456,0',run_mode=RUN_NONINTERACTIVE)
	if frame==6:	# " Frame [Painting At06A_2017] "
		gimp.pdb.plug_in_gmic_qt( image, layer, 1, 2, 'samj_At06A_2017_frame_painting 10,0.4,6,' + str(rgb) + ',2,400,50,10,1,0.5,123456,35,1,1,1,0,100',run_mode=RUN_NONINTERACTIVE)		
	if frame==7:	# " Frame [Regular]
		gimp.pdb.plug_in_gmic_qt( image, layer, 1, 2, 'fx_frame 0,100,0,100,10,10,' + str(rgb) + ',255,1,255,255,255,255',run_mode=RUN_NONINTERACTIVE)
	if frame==8:	# " reliefFrame "
		gimp.pdb.plug_in_gmic_qt( image, layer, 1, 2, 'cl_reliefFrame 2,10,1.5,75,1,60,0,0,0,0,192,192,192,0,0,0',run_mode=RUN_NONINTERACTIVE)
		
	if frame==9:	# " Frame [Round] "
		gimp.pdb.plug_in_gmic_qt( image, layer, 1, 2, 'fx_frame_round 6,20,0.1,0,' + str(rgb) + ',255,0,0.1,3',run_mode=RUN_NONINTERACTIVE)		
	if frame==10:	# " Old Photograph "
		gimp.pdb.plug_in_gmic_qt( image, layer, 1, 2, 'fx_old_photo 200,50,85',run_mode=RUN_NONINTERACTIVE)
	if frame==11:	# " Polaroid "
		gimp.pdb.plug_in_gmic_qt( image, layer, 1, 2, 'fx_polaroid 10,20,0,0,3,1,0,255,100,100',run_mode=RUN_NONINTERACTIVE)	
	if frame==12:	# " Vignette "
		gimp.pdb.plug_in_gmic_qt( image, layer, 1, 2, 'fx_vignette 70,70,95,' + str(rgb) + ',255',run_mode=RUN_NONINTERACTIVE)
		
	active_layer_f = pdb.gimp_image_get_active_layer(image)
	active_layer_f.name ="Add Frame " + str(frame_command[frame])
	
# centered all layers
	for layer in image.layers:		
		wh = pdb.gimp_drawable_width(layer)
		hh = pdb.gimp_drawable_height(layer)
		w = pdb.gimp_image_width(image)
		h = pdb.gimp_image_height(image)
		pdb.gimp_image_set_active_layer(image,layer)
		centerx=w/2
		centery=h/2
		adw=wh/2
		adh=hh/2
		sw=centerx - adw
		sh=centery - adh		
		pdb.gimp_layer_set_offsets(layer, sw, sh)
# -----------------------------------------------------	
		
	if flatten:
	    image.merge_visible_layers(CLIP_TO_IMAGE)
		
### Info
	# Text layer
	p = "  G'MIC Command: " + "\n " + "  GC 1  = " + str(gmic_command[gmic1]) + "\n " + "  GC 2  = " + str(gmic_command[gmic2]) + "\n " + "  Frame = " + str(frame_command[frame])
	# Layer name
	q = str(gmic_command[gmic1]) + " + " + str(gmic_command[gmic2])
	
	info = pdb.gimp_text_fontname(image, None,  0.0, 0.0, p+" ", 0, 1, 16, 1, "Arial")
	pdb.gimp_text_layer_set_color(info, (0,0,0))
	gimp_copy = info.copy(0)
	image.add_layer(gimp_copy, -1)
	pdb.gimp_edit_clear(gimp_copy)
	pdb.gimp_context_set_foreground((255,255,255))
	pdb.gimp_edit_fill(gimp_copy,2)
	pdb.gimp_image_raise_item_to_top(image, gimp_copy)	
	pdb.gimp_image_raise_item_to_top(image, info)
	ren = pdb.gimp_image_merge_down(image, info, 0)
	#ren.name = ren.name.replace("#1","")
	pdb.gimp_item_set_name(ren, "Info G'MIC Command")	
		
	# display results
	gimp.Display(image)


register(
	"python_fu_gmic_exercise",
	"Creating from active layer new artistic image from use G'MIC-QT. Ver.0.6",
	"Creates logo with specified text and image as 3 command G'mic",
	"MareoQ",
	"MrQ",
	"2020",
	"G'MIC-QT Exercise...",
	"",
	[
		(PF_IMAGE,    "image", "Input image", None),
		(PF_DRAWABLE, "layer", "Input layer", None), 
		(PF_OPTION,   "options", "Use to:", 1, ["Text Logo", "Image"]),
		(PF_TEXT, 	  "text", ("Text:"), "GIMP \nSCRIPTS \nNET ",0 ,{'tab':1}),
		(PF_SPINNER,  "margin", "Add border (px):", 50, (0, 500, 1),{'tab':1}),
		(PF_FONT,     "font", "Font:", "Arial",0 ,{'tab':1}),
		(PF_OPTION,   "Direction", "Direction:", 0, ["From left to right",  "From right to left", "Vertical, right to left (mixed orientation)", "Vertical, right to left (upright orientation)", "Vertical, left to right (mixed orientation)", "Vertical, left to right (upright orientation)\n [* add a space at the end of the line !!!]"],{'tab':1}),
		(PF_OPTION,   "just", "Justify:", 2, ["LEFT","RIGHT","CENTER","FILL"] ,{'tab':1}),
		(PF_SPINNER,  "letter_spacing", "Letter Spacing:", 0, (-100, 100, 1, 5, 0, 0) ,{'tab':1}),
		(PF_SPINNER,  "line_spacing", "Line Spacing:", 0, ( -100, 100, 1, 5, 0, 0) ,{'tab':1}),
		
		(PF_SPINNER,  "font_size", "Font size (px):", 50, (0, 500, 1),{'tab':1}),		
		#(PF_INT,      "font_size", "Font size (px):", 150 ,{'tab':1}),
		(PF_OPTION,   "fill_t", "Filling Text", 0,("Color","Patterns") ,{'tab':1}),		
		(PF_COLOR,    "text_color", " - color:", (225, 225, 35),0, {'tab':1}),
		(PF_PATTERN,  "text_pattern", " - pattern:", "",0, {'tab':1}),
		(PF_OPTION,   "fill_b", "Filling Background:", 0,("Color","Patterns") ,{'tab':1}),
		(PF_COLOR,    "background_color", " - color:", (50, 25, 225),0, {'tab':1}),
		(PF_PATTERN,  "b_pattern", " - pattern", "",0, {'tab':1}),		
		(PF_SLIDER,   "background_opacity", "Background opacity:", 80, (10, 100, 1) ,{'tab':1}),
		
		(PF_OPTION,   "gmic1", "G'MIC-QT\nCommand 1: ", 0,gmic_command),
		(PF_OPTION,   "gmic2", "G'MIC-QT\nCommand 2: ", 0,gmic_command),		
		(PF_OPTION,  "frame", "Add frame:", 2, [
			" RANDOM ", 
			" DON'T USE ",
			" Frame [Blur] ",
			" Frame [Fuzzy] ",
			" * Frame [Mirror] ",
			" Frame [Painting] ", 
			" Frame [Painting At06A_2017]",
			" Frame [Regular] ",
			" * Frame [Relief] ",
			" Frame [Round] ",
			" * Old Photograph ",
			" * Polaroid ",
			" Vignette "]),		
		(PF_COLOR, "frame_color", "Frame color:\n ( * = not use color)", (127, 127, 127),0,),
		(PF_BOOL,   "flatten",      "Merge layers:", False,0,),
		(PF_TAB    , "ptab"    , "tabs", 0, ("G'MIC", "TEXT",)),			
	
		],
		[],
		python_fu_gmic_exercise,
		menu="<Image>/Filters",
		#menu=
		)

main()

